//*****************************************************************************
//
//	Routinen fuer das Lesen und Schreiben ueber I2C-Bus
//
//  Takt = 10kHz    geht 
//*****************************************************************************

#include <iom128v.h>
#include "i2ch.h"
#include "m128int.h"
#include "watchdog.h"


//        CPU_CLK - 16 * SCL               in kHz   
// TWBR = -------------------
//        2 * SCL * (TWPS +1)       


//CPU_CLK 16000 kHz, SCL 10 kHz, TWPS 3  => FWBR 198

//FWBR 
#define BITRATE        0x0c
#define TWI_PRESCALE   0x03




#pragma interrupt_handler I2C_INT:TWI_INT

unsigned char I2C_break = 0,I2C_Errorflag = 0,I2C_State = 0;


//*****************************************************************************
//	I2C - TWI_INTERRUPT
//  Input:	-
//	Output:	-
//*****************************************************************************
void I2C_INT(void)
{
   I2C_break = 1;
}


//*****************************************************************************
//	I2C - Warten  
//  Input:	-
//	Output:	-
//*****************************************************************************
void I2C_wait(void)
{
unsigned int uc_cnt = 0xFFF;

   while (!I2C_break)
   {
     if ( (TWCR & I2C_TWINT) == I2C_TWINT) I2C_break = 1;
	 if (uc_cnt == 0)
	  {
	    I2C_break = 1;
	  }
	 uc_cnt--;
   }
   I2C_break = 0;
}


//*****************************************************************************
//	I2C - Init des I2C-Bus
//  Input:	-
//	Output:	-
//*****************************************************************************
void I2C_init(void)
{
   I2C_break = 0;
   TWSR &= 0xfc;		
   TWSR |= TWI_PRESCALE;
   TWBR = BITRATE;		
}	 


//*****************************************************************************
//	I2C - Senden eines Adressbytes
//  Input:	1 DB
//	Output:	-
//*****************************************************************************
void I2C_transbyteADR(unsigned char uc_byte) 
{
   I2C_State = I2C_STATE_TBADR;

   TWSR &= 0x02; 
   TWDR = uc_byte;
   TWCR = I2C_TWINT | I2C_TWEN;
   I2C_wait();
}


//*****************************************************************************
//	Senden eines Bytes
//  Input:	1 DB
//	Output:	-
//*****************************************************************************
void I2C_transbyte(unsigned char uc_byte) 
{
   I2C_State = I2C_STATE_TBDB;

   TWSR &= 0x02; 
   TWDR = uc_byte;
   TWCR = I2C_TWINT | I2C_TWEN;
   I2C_wait();
}


//*****************************************************************************
//	I2C - Empfangen eines Bytes
//  Input:	0 = 1..n-tes DB (Acknolege senden), 1 = letztes DB (kein Acknolege)
//	Output:	1DB gelesen
//*****************************************************************************
unsigned char I2C_GetByte(unsigned char uc_lastbyte) 
{
   TWSR &= 0x02;
   if (uc_lastbyte) TWCR = I2C_TWINT | I2C_TWEN;
               else TWCR = I2C_TWINT | I2C_TWACK | I2C_TWEN;

   I2C_wait();
   return(TWDR);
}


//*****************************************************************************
//	I2C - Empfangen eines Bytes, 1.. n-tes DB (Acknolege senden)
//  Input:	-
//	Output:	1DB gelesen
//*****************************************************************************
unsigned char I2C_recbyte(void)
{
   I2C_State = I2C_STATE_RB;

   return(I2C_GetByte(0));
}


//*****************************************************************************
//	I2C - Empfangen eines Bytes, letztes DB, (kein Acknolege senden)
//  Input:	-
//	Output:	1DB gelesen
//*****************************************************************************
unsigned char I2C_recbyteend(void)
{
   I2C_State = I2C_STATE_RBEND;

   return(I2C_GetByte(1));
}


//*****************************************************************************
//	I2C - Start-condition
//  Input:	-
//	Output:	-
//*****************************************************************************
void I2C_constart(void) 
{
   I2C_State = I2C_STATE_CONSTART;

   I2C_break = 0;
   TWSR &= 0x02;		// Statusregister ruecksetzen
   TWCR = I2C_TWINT | I2C_TWEN | I2C_TWSTART;
   I2C_wait();
}


//*****************************************************************************
//	I2C - Stop-condition
//  Input:	-
//	Output:	-
//*****************************************************************************
void I2C_constop(void) 
{
   I2C_State = I2C_STATE_CONSTOP;

   TWSR &= 0x02; 
   TWCR = I2C_TWINT | I2C_TWEN | I2C_TWSTOP;
   I2C_wait();
}


//*****************************************************************************
//	I2C - BUS Stop
//  Input:	-
//	Output:	-
//*****************************************************************************
void I2C_BUSSTOP(void) 
{
   I2C_State = I2C_STATE_CLEAR;

   TWCR = 0x90; 
}


//*****************************************************************************
//	I2C - BUS Error (Arbeitsschritt)
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char I2C_ERROR(void) 
{
unsigned char uc_Status,uc_Error,uc_Ergebnis;

   uc_Error = 1;
   uc_Status = TWSR & 0xF8;
   
   switch(I2C_State)
   {
   case I2C_STATE_CLEAR:    uc_Error = 0;
                            if (uc_Error == 1) return(I2C_STATE_CLEAR);
                            break;
   case I2C_STATE_CONSTART: if (uc_Status == MSTAT_START) uc_Error = 0;    
                            if (uc_Status == MSTAT_REPEAT_START) uc_Error = 0;						
                            if (uc_Error == 1) return(I2C_STATE_CONSTART);
                            break;
   case I2C_STATE_TBADR:    if (uc_Status == MSTAT_WSLA_ACK) uc_Error = 0;    
                            if (uc_Status == MSTAT_WSLA_NACK) uc_Error = 0;
                            if (uc_Status == MSTAT_RSLA_ACK) uc_Error = 0;    
                            if (uc_Status == MSTAT_RSLA_NACK) uc_Error = 0;                            
                            if (uc_Status == SSTAT_WSLA_ACK) uc_Error = 0;    
                            if (uc_Status == SSTAT_WSLA_NACK) uc_Error = 0;
                            if (uc_Status == SSTAT_RSLA_ACK) uc_Error = 0;    
                            if (uc_Status == SSTAT_RSLA_NACK) uc_Error = 0;                            
                            if (uc_Status == SSTAT_GCALL) uc_Error = 0;                            
							if (uc_Error == 1) return(I2C_STATE_TBADR);
                            break;
   case I2C_STATE_TBDB:     if (uc_Status == MSTAT_WDB_ACK) uc_Error = 0;    
                            if (uc_Status == MSTAT_WDB_NACK) uc_Error = 0;
                            if (uc_Status == SSTAT_PWSLA_ACK) uc_Error = 0;    
                            if (uc_Status == SSTAT_PWSLA_NACK) uc_Error = 0;                            
                            if (uc_Status == SSTAT_RDBGCALL_ACK) uc_Error = 0;    
                            if (uc_Status == SSTAT_RDBGCALL_NACK) uc_Error = 0;   				                         
							if (uc_Error == 1) return(I2C_STATE_TBDB);
                            break;
   case I2C_STATE_RB:       if (uc_Status == MSTAT_RDB_ACK) uc_Error = 0;    
                            if (uc_Status == MSTAT_RDB_NACK) uc_Error = 0;
							if (uc_Status == SSTAT_RDB_ACK) uc_Error = 0;    
                            if (uc_Status == SSTAT_RDB_NACK) uc_Error = 0;
							if (uc_Error == 1) return(I2C_STATE_RB);
                            break;
   case I2C_STATE_RBEND:    if (uc_Status == MSTAT_RDB_ACK) uc_Error = 0;    
                            if (uc_Status == MSTAT_RDB_NACK) uc_Error = 0;
                            if (uc_Status == SSTAT_RDB_ACK) uc_Error = 0;    
                            if (uc_Status == SSTAT_RDB_NACK) uc_Error = 0;
                            if (uc_Status == SSTAT_RLASTDB_ACK) uc_Error = 0;
							if (uc_Error == 1) return(I2C_STATE_RBEND);
                            break;
   case I2C_STATE_CONSTOP:  uc_Error = 0;    
                            if (uc_Error == 1) return(I2C_STATE_CONSTOP);
                            break;
   }

   return(0); //Keine Fehler aufgetreten
}


